/*
 * Copyright (c) 1994, 1995, 1999 Vectaport Inc.
 * Copyright (c) 1990, 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * FrameViewer - a Viewer with support for frame operations
 */

#ifndef frame_viewer_h
#define frame_viewer_h

#include <OverlayUnidraw/ovviewer.h>

#include <IV-2_6/_enter.h>

class FrameEditor;
class OverlayView;

//: OverlayViewer specialized for use with FrameEditor.
class FrameViewer : public OverlayViewer {
public:
    FrameViewer(
        Editor*, GraphicView*, UPage*, Grid* = nil, 
        Coord = 0, Coord = 0, Orientation = Normal,
	Alignment = Center, Zooming = Continuous
    );

    void Update();
    virtual void SetGraphicView(GraphicView*);

    virtual GraphicView* GetCurrentGraphicView();
    // allow for other than the top-level graphic view

    FrameEditor* GetFrameEditor() { return (FrameEditor*) GetEditor(); }

    virtual OverlayView* GetCurrent();
};

#include <IV-2_6/_leave.h>

#endif
