/*
 * Copyright (c) 1998 R.B. Kissh & Associates
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef xt_ud_session_h
#define xt_ud_session_h

#include "xtsession.h"

#include <OverlayUnidraw/ovunidraw.h>

class Creator;


class XtUnidraw : public OverlayUnidraw {
public:
    XtUnidraw(Catalog*, World*);

    static XtUnidraw& instance();

    virtual void Open(Editor*);
    virtual void Open(Editor*, Widget widget);
    virtual void Close(Editor*);

    virtual void workProc();
};



class XtUdSession : public XtSession {
public:
    static initialize(
        const char*, int& argc, char** argv, XDisplay*, 
        const OptionDesc* = nil, const PropertyData* = nil
    );

    static initialize(
        const char*, int& argc, char** argv, XDisplay*, 
        Catalog*, const OptionDesc* = nil, const PropertyData* = nil
    );


    XtUdSession(
      const char*, int& argc, char** argv, XtAppContext, Catalog*, 
      const OptionDesc* = nil, const PropertyData* = nil, Display* = nil
    );

    virtual void workProc();

};

#endif
