/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/*
 * Use current by default for ambiguous names.
 */

#undef Adjuster
#undef Border
#undef Box
#undef Button
#undef PushButton
#undef RadioButton
#undef Center
#undef Coord
#undef Deck
#undef Dialog
#undef FileBrowser
#undef FileChooser
#undef Glue
#undef HGlue
#undef VGlue
#undef Menu
#undef MenuBar
#undef MenuItem
#undef Mover
#undef UpMover
#undef DownMover
#undef LeftMover
#undef RightMover
#undef Panner
#undef Scroller
#undef HScroller
#undef VScroller
#undef ScrollBar
#undef HScrollBar
#undef VScrollBar
#undef Slider

#define Adjuster _lib_iv(Adjuster)
#define Border _lib_iv(Border)
#define Box _lib_iv(Box)
#define Button _lib_iv(Button)
#define PushButton _lib_iv(PushButton)
#define RadioButton _lib_iv(RadioButton)
#define Center _lib_iv(Center)
#define Coord _lib_iv(Coord)
#define Deck _lib_iv(Deck)
#define Dialog _lib_iv(Dialog)
#define FileBrowser _lib_iv(FileBrowser)
#define FileChooser _lib_iv(FileChooser)
#define Glue _lib_iv(Glue)
#define HGlue _lib_iv(HGlue)
#define VGlue _lib_iv(VGlue)
#define Menu _lib_iv(Menu)
#define MenuBar _lib_iv(MenuBar)
#define MenuItem _lib_iv(MenuItem)
#define Mover _lib_iv(Mover)
#define UpMover _lib_iv(UpMover)
#define DownMover _lib_iv(DownMover)
#define LeftMover _lib_iv(LeftMover)
#define RightMover _lib_iv(RightMover)
#define Panner _lib_iv(Panner)
#define Scroller _lib_iv(Scroller)
#define HScroller _lib_iv(HScroller)
#define VScroller _lib_iv(VScroller)
#define ScrollBar _lib_iv(ScrollBar)
#define HScrollBar _lib_iv(HScrollBar)
#define VScrollBar _lib_iv(VScrollBar)
#define Slider _lib_iv(Slider)
