#undef boolean

#define boolean _lib_os(boolean)
#define u_char _lib_os(u_char)
#define CopyString _lib_os(CopyString)
#define Directory _lib_os(Directory)
#define DirectoryImpl _lib_os(DirectoryImpl)
#define File _lib_os(File)
#define FileInfo _lib_os(FileInfo)
#define Host _lib_os(Host)
#define InputFile _lib_os(InputFile)
#define List _lib_os(List)
#define Math _lib_os(Math)
#define Memory _lib_os(Memory)
#define NullTerminatedString _lib_os(NullTerminatedString)
#define OutputFile _lib_os(OutputFile)
#define PtrList _lib_os(PtrList)
#define StdInput _lib_os(StdInput)
#define StdOutput _lib_os(StdOutput)
#define String _lib_os(String)
#define Table _lib_os(Table)
#define Table2 _lib_os(Table2)
#define UniqueString _lib_os(UniqueString)
#define UniqueStringPool _lib_os(UniqueStringPool)
#define UniqueStringTable _lib_os(UniqueStringTable)
