#undef Alignment
#undef ColorIntensity
#undef ColorOp
#undef Coord
#undef EventType
#undef EventButton
#undef GlyphBreakType
#undef GlyphIndex
#undef IntCoord
#undef PixelCoord
#undef Selection
#undef Action
#undef ActiveHandler
#undef Adjustable
#undef Adjuster
#undef Aggregate
#undef AggregateInfo_List
#undef Align
#undef Allocation
#undef AllocationInfo
#undef AllocationTable
#undef AllocationTableImpl
#undef Allotment
#undef ApplicationWindow
#undef ArrayCompositor
#undef ArrowSubclassName
#undef Background
#undef Banner
#undef Bevel
#undef BevelFrame
#undef Bitmap
#undef BitmapRep
#undef Border
#undef BorderFrame
#undef Box
#undef BoxCanonical
#undef BoxElement
#undef BoxImpl
#undef Break
#undef Break_List
#undef Browser
#undef BrowserList
#undef Brush
#undef BrushImpl
#undef BrushRep
#undef Button
#undef ButtonList
#undef ButtonState
#undef Canvas
#undef CanvasDamage
#undef CanvasRep
#undef Card
#undef CenterLayout
#undef Character
#undef CheckBox
#undef ChoiceItem
#undef ClippingStack
#undef Color
#undef ColorImpl
#undef ColorRep
#undef ColorTable
#undef CompletionEditor
#undef Composition
#undef CompositionComponent_List
#undef Compositor
#undef Control
#undef ControlState
#undef Cursor
#undef CursorRep
#undef CursorRepBitmap
#undef CursorRepData
#undef CursorRepFont
#undef CursorRepXFont
#undef DamageList
#undef DebugGlyph
#undef Deck
#undef Dialog
#undef DialogKit
#undef Display
#undef DisplayRep
#undef DownMover
#undef Enlarger
#undef Event
#undef EventRep
#undef Extension
#undef FBDirectory
#undef FieldEditor
#undef FieldEditorAction
#undef FieldEditorImpl
#undef FileBrowser
#undef FileBrowserImpl
#undef FileChooser
#undef FileChooserAction
#undef FileChooserImpl
#undef FixedLayout
#undef Font
#undef FontBoundingBox
#undef FontFamily
#undef FontFamilyImpl
#undef FontFamilyRep
#undef FontImpl
#undef FontRep
#undef Frame
#undef GLContext
#undef GLContextImpl
#undef GLWindow
#undef Glue
#undef Glyph
#undef GrabList
#undef Group
#undef GrowingBSpline
#undef GrowingClosedBSpline
#undef GrowingMultiLine
#undef GrowingPolygon
#undef GrowingVertices
#undef HBorder
#undef HBox
#undef HGlue
#undef HRule
#undef HScrollBar
#undef HScroller
#undef Handler
#undef Hit
#undef HitImpl
#undef IconWindow
#undef Image
#undef InputHandler
#undef InputHandlerImpl
#undef Interactor
#undef InteractorHandler
#undef InteractorWindow
#undef KnownFonts
#undef LRComposition
#undef LRMarker
#undef Label
#undef Layout
#undef LayoutKit
#undef LeftMover
#undef MFDialogKit
#undef MFKit
#undef MFKitImpl
#undef Macro
#undef ManagedWindow
#undef ManagedWindowHintInfo
#undef ManagedWindowRep
#undef MarginFrame
#undef MarginLayout
#undef MatchEditor
#undef Menu
#undef MenuBar
#undef MenuImpl
#undef MenuItem
#undef Message
#undef MonoGlyph
#undef MonoScene
#undef Mover
#undef NaturalLayout
#undef Observable
#undef Observer
#undef ObserverList
#undef OptionDesc
#undef PSFont
#undef PSFontImpl
#undef Page
#undef PageInfo_List
#undef Painter
#undef PainterRep
#undef Panner
#undef Patch
#undef PathRenderInfo
#undef Pattern
#undef PatternRep
#undef Perspective
#undef Placement
#undef PolyGlyph
#undef PolyGlyphImpl
#undef PopupMenu
#undef PopupWindow
#undef Printer
#undef PrinterRep
#undef PropertyData
#undef PulldownMenu
#undef PullrightMenu
#undef PushButton
#undef RGBTable
#undef RadioButton
#undef Raster
#undef RasterRep
#undef Reducer
#undef Regexp
#undef ReqErr
#undef Requirement
#undef Requisition
#undef Resource
#undef RightMover
#undef RotatingLine
#undef RotatingLineList
#undef RotatingRect
#undef RubberAxis
#undef RubberCircle
#undef RubberClosedSpline
#undef RubberEllipse
#undef RubberGroup
#undef RubberHandles
#undef RubberLine
#undef RubberList
#undef RubberPointList
#undef RubberRect
#undef RubberSpline
#undef RubberSquare
#undef RubberVertex
#undef Rubberband
#undef Rule
#undef SMFKit
#undef SMFKitImpl
#undef ScalingLine
#undef ScalingLineList
#undef ScalingRect
#undef Scene
#undef ScrollBar
#undef ScrollBox
#undef Scroller
#undef SelectionHandler
#undef SelectionList
#undef SelectionManager
#undef SelectionManagerRep
#undef Sensor
#undef Session
#undef SessionRep
#undef Shadow
#undef ShadowFrame
#undef Shape
#undef ShowFrame
#undef SimpleCompositor
#undef Slider
#undef SliderImpl
#undef SlidingEllipse
#undef SlidingLine
#undef SlidingLineList
#undef SlidingPointList
#undef SlidingRect
#undef Stencil
#undef Stepper
#undef StepperSubclassName
#undef StretchingRect
#undef StringBrowser
#undef StringChooser
#undef StringEditor
#undef Style
#undef StyleRep
#undef Subject
#undef SubjectRep
#undef Superpose
#undef TBComposition
#undef TBScrollBox
#undef TBScrollBoxImpl
#undef TGlue
#undef TIFFRaster
#undef TSolver
#undef Target
#undef TeXCompositor
#undef Telltale
#undef TelltaleGroup
#undef TelltaleState
#undef TextBuffer
#undef TextButton
#undef TextDisplay
#undef TextEditor
#undef TextRenderInfo
#undef Tile
#undef TileFirstAligned
#undef TileReversed
#undef TileReversedFirstAligned
#undef TitleFrame
#undef TopLevelWindow
#undef TransformFitter
#undef TransformSetter
#undef Transformer
#undef TransformerStack
#undef TransientWindow
#undef Tray
#undef TrayElement
#undef UpMover
#undef VBorder
#undef VBox
#undef VGlue
#undef VRule
#undef VScrollBar
#undef VScroller
#undef VariableLayout
#undef ViewList
#undef Viewport
#undef WidgetKit
#undef WidgetKitImpl
#undef Window
#undef WindowRep
#undef WindowTable
#undef WindowVisual
#undef World
#undef XSlider
#undef XYMarker
#undef XYSlider
#undef YSlider
#undef Zoomer
