/*
 * Copyright (c) 1990, 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Object creator class.
 */

#ifndef unidraw_creator_h
#define unidraw_creator_h

#include <Unidraw/globals.h>

#define CREATE(classname, instream, objmap, objid) { \
    classname* o = new classname; \
    if (objmap != nil) objmap->Register(o, objid); \
    o->Read(instream); \
    return o; \
}

class ObjectMap;
#include <iosfwd>

//: component factory class
// <a href=../man3.1/Creator.html>man page</a>
class Creator {
public:
    Creator();

    virtual void* Create(                       // for subjects
        ClassId, istream&, ObjectMap* = nil, int = 0
    );
    virtual void* Create(ClassId);              // for views

    static Creator* instance();
    static void instance(Creator*);

protected:
    static Creator* _instance;
};

#endif
