#define Alignment _lib_iv(Alignment)
#define ColorIntensity _lib_iv(ColorIntensity)
#define ColorOp _lib_iv(ColorOp)
#define Coord _lib_iv(Coord)
#define EventType _lib_iv(EventType)
#define EventButton _lib_iv(EventButton)
#define GlyphBreakType _lib_iv(GlyphBreakType)
#define GlyphIndex _lib_iv(GlyphIndex)
#define IntCoord _lib_iv(IntCoord)
#define PixelCoord _lib_iv(PixelCoord)
#define FloatCoord _lib_iv(FloatCoord)
#define Selection _lib_iv(Selection)
#define Action _lib_iv(Action)
#define ActiveHandler _lib_iv(ActiveHandler)
#define Adjustable _lib_iv(Adjustable)
#define Adjuster _lib_iv(Adjuster)
#define Aggregate _lib_iv(Aggregate)
#define AggregateInfo_List _lib_iv(AggregateInfo_List)
#define Align _lib_iv(Align)
#define Allocation _lib_iv(Allocation)
#define AllocationInfo _lib_iv(AllocationInfo)
#define AllocationTable _lib_iv(AllocationTable)
#define AllocationTableImpl _lib_iv(AllocationTableImpl)
#define Allotment _lib_iv(Allotment)
#define ApplicationWindow _lib_iv(ApplicationWindow)
#define ArrayCompositor _lib_iv(ArrayCompositor)
#define ArrowSubclassName _lib_iv(ArrowSubclassName)
#define Background _lib_iv(Background)
#define Banner _lib_iv(Banner)
#define Bevel _lib_iv(Bevel)
#define BevelFrame _lib_iv(BevelFrame)
#define Bitmap _lib_iv(Bitmap)
#define BitmapRep _lib_iv(BitmapRep)
#define Border _lib_iv(Border)
#define BorderFrame _lib_iv(BorderFrame)
#define Box _lib_iv(Box)
#define BoxCanonical _lib_iv(BoxCanonical)
#define BoxElement _lib_iv(BoxElement)
#define BoxImpl _lib_iv(BoxImpl)
#define Break _lib_iv(Break)
#define Break_List _lib_iv(Break_List)
#define Browser _lib_iv(Browser)
#define BrowserList _lib_iv(BrowserList)
#define Brush _lib_iv(Brush)
#define BrushImpl _lib_iv(BrushImpl)
#define BrushRep _lib_iv(BrushRep)
#define Button _lib_iv(Button)
#define ButtonList _lib_iv(ButtonList)
#define ButtonState _lib_iv(ButtonState)
#define Canvas _lib_iv(Canvas)
#define CanvasDamage _lib_iv(CanvasDamage)
#define CanvasRep _lib_iv(CanvasRep)
#define Card _lib_iv(Card)
#define CenterLayout _lib_iv(CenterLayout)
#define Character _lib_iv(Character)
#define CheckBox _lib_iv(CheckBox)
#define ChoiceItem _lib_iv(ChoiceItem)
#define ClippingStack _lib_iv(ClippingStack)
#define Color _lib_iv(Color)
#define ColorImpl _lib_iv(ColorImpl)
#define ColorRep _lib_iv(ColorRep)
#define ColorTable _lib_iv(ColorTable)
#define CompletionEditor _lib_iv(CompletionEditor)
#define Composition _lib_iv(Composition)
#define CompositionComponent_List _lib_iv(CompositionComponent_List)
#define Compositor _lib_iv(Compositor)
#define Control _lib_iv(Control)
#define ControlState _lib_iv(ControlState)
#define Cursor _lib_iv(Cursor)
#define CursorRep _lib_iv(CursorRep)
#define CursorRepBitmap _lib_iv(CursorRepBitmap)
#define CursorRepData _lib_iv(CursorRepData)
#define CursorRepFont _lib_iv(CursorRepFont)
#define CursorRepXFont _lib_iv(CursorRepXFont)
#define DamageList _lib_iv(DamageList)
#define DebugGlyph _lib_iv(DebugGlyph)
#define Deck _lib_iv(Deck)
#define Dialog _lib_iv(Dialog)
#define DialogKit _lib_iv(DialogKit)
#define Display _lib_iv(Display)
#define DisplayRep _lib_iv(DisplayRep)
#define DownMover _lib_iv(DownMover)
#define Enlarger _lib_iv(Enlarger)
#define Event _lib_iv(Event)
#define EventRep _lib_iv(EventRep)
#define Extension _lib_iv(Extension)
#define FBDirectory _lib_iv(FBDirectory)
#define FieldEditor _lib_iv(FieldEditor)
#define FieldEditorAction _lib_iv(FieldEditorAction)
#define FieldEditorImpl _lib_iv(FieldEditorImpl)
#define FileBrowser _lib_iv(FileBrowser)
#define FileBrowserImpl _lib_iv(FileBrowserImpl)
#define FileChooser _lib_iv(FileChooser)
#define FileChooserAction _lib_iv(FileChooserAction)
#define FileChooserImpl _lib_iv(FileChooserImpl)
#define FixedLayout _lib_iv(FixedLayout)
#define Font _lib_iv(Font)
#define FontBoundingBox _lib_iv(FontBoundingBox)
#define FontFamily _lib_iv(FontFamily)
#define FontFamilyImpl _lib_iv(FontFamilyImpl)
#define FontFamilyRep _lib_iv(FontFamilyRep)
#define FontImpl _lib_iv(FontImpl)
#define FontRep _lib_iv(FontRep)
#define Frame _lib_iv(Frame)
#define GLContext _lib_iv(GLContext)
#define GLContextImpl _lib_iv(GLContextImpl)
#define GLWindow _lib_iv(GLWindow)
#define Glue _lib_iv(Glue)
#define Glyph _lib_iv(Glyph)
#define GrabList _lib_iv(GrabList)
#define Group _lib_iv(Group)
#define GrowingBSpline _lib_iv(GrowingBSpline)
#define GrowingClosedBSpline _lib_iv(GrowingClosedBSpline)
#define GrowingMultiLine _lib_iv(GrowingMultiLine)
#define GrowingPolygon _lib_iv(GrowingPolygon)
#define GrowingVertices _lib_iv(GrowingVertices)
#define HBorder _lib_iv(HBorder)
#define HBox _lib_iv(HBox)
#define HGlue _lib_iv(HGlue)
#define HRule _lib_iv(HRule)
#define HScrollBar _lib_iv(HScrollBar)
#define HScroller _lib_iv(HScroller)
#define Handler _lib_iv(Handler)
#define Hit _lib_iv(Hit)
#define HitImpl _lib_iv(HitImpl)
#define IconWindow _lib_iv(IconWindow)
#define Image _lib_iv(Image)
#define InputHandler _lib_iv(InputHandler)
#define InputHandlerImpl _lib_iv(InputHandlerImpl)
#define Interactor _lib_iv(Interactor)
#define InteractorHandler _lib_iv(InteractorHandler)
#define InteractorWindow _lib_iv(InteractorWindow)
#define KnownFonts _lib_iv(KnownFonts)
#define LRComposition _lib_iv(LRComposition)
#define LRMarker _lib_iv(LRMarker)
#define Label _lib_iv(Label)
#define Layout _lib_iv(Layout)
#define LayoutKit _lib_iv(LayoutKit)
#define LeftMover _lib_iv(LeftMover)
#define MFDialogKit _lib_iv(MFDialogKit)
#define MFKit _lib_iv(MFKit)
#define MFKitImpl _lib_iv(MFKitImpl)
#define Macro _lib_iv(Macro)
#define ManagedWindow _lib_iv(ManagedWindow)
#define ManagedWindowHintInfo _lib_iv(ManagedWindowHintInfo)
#define ManagedWindowRep _lib_iv(ManagedWindowRep)
#define MarginFrame _lib_iv(MarginFrame)
#define MarginLayout _lib_iv(MarginLayout)
#define MatchEditor _lib_iv(MatchEditor)
#define Menu _lib_iv(Menu)
#define MenuBar _lib_iv(MenuBar)
#define MenuImpl _lib_iv(MenuImpl)
#define MenuItem _lib_iv(MenuItem)
#define Message _lib_iv(Message)
#define MonoGlyph _lib_iv(MonoGlyph)
#define MonoScene _lib_iv(MonoScene)
#define Mover _lib_iv(Mover)
#define NaturalLayout _lib_iv(NaturalLayout)
#define Observable _lib_iv(Observable)
#define Observer _lib_iv(Observer)
#define ObserverList _lib_iv(ObserverList)
#define OptionDesc _lib_iv(OptionDesc)
#define PSFont _lib_iv(PSFont)
#define PSFontImpl _lib_iv(PSFontImpl)
#define Page _lib_iv(Page)
#define PageInfo_List _lib_iv(PageInfo_List)
#define Painter _lib_iv(Painter)
#define PainterRep _lib_iv(PainterRep)
#define Panner _lib_iv(Panner)
#define Patch _lib_iv(Patch)
#define PathRenderInfo _lib_iv(PathRenderInfo)
#define Pattern _lib_iv(Pattern)
#define PatternRep _lib_iv(PatternRep)
#define Perspective _lib_iv(Perspective)
#define Placement _lib_iv(Placement)
#define PolyGlyph _lib_iv(PolyGlyph)
#define PolyGlyphImpl _lib_iv(PolyGlyphImpl)
#define PopupMenu _lib_iv(PopupMenu)
#define PopupWindow _lib_iv(PopupWindow)
#define Printer _lib_iv(Printer)
#define PrinterRep _lib_iv(PrinterRep)
#define PropertyData _lib_iv(PropertyData)
#define PulldownMenu _lib_iv(PulldownMenu)
#define PullrightMenu _lib_iv(PullrightMenu)
#define PushButton _lib_iv(PushButton)
#define RGBTable _lib_iv(RGBTable)
#define RadioButton _lib_iv(RadioButton)
#define Raster _lib_iv(Raster)
#define RasterRep _lib_iv(RasterRep)
#define Reducer _lib_iv(Reducer)
#define Regexp _lib_iv(Regexp)
#define ReqErr _lib_iv(ReqErr)
#define Requirement _lib_iv(Requirement)
#define Requisition _lib_iv(Requisition)
#define Resource _lib_iv(Resource)
#define RightMover _lib_iv(RightMover)
#define RotatingLine _lib_iv(RotatingLine)
#define RotatingLineList _lib_iv(RotatingLineList)
#define RotatingRect _lib_iv(RotatingRect)
#define RubberAxis _lib_iv(RubberAxis)
#define RubberCircle _lib_iv(RubberCircle)
#define RubberClosedSpline _lib_iv(RubberClosedSpline)
#define RubberEllipse _lib_iv(RubberEllipse)
#define RubberGroup _lib_iv(RubberGroup)
#define RubberHandles _lib_iv(RubberHandles)
#define RubberLine _lib_iv(RubberLine)
#define RubberList _lib_iv(RubberList)
#define RubberPointList _lib_iv(RubberPointList)
#define RubberRect _lib_iv(RubberRect)
#define RubberSpline _lib_iv(RubberSpline)
#define RubberSquare _lib_iv(RubberSquare)
#define RubberVertex _lib_iv(RubberVertex)
#define Rubberband _lib_iv(Rubberband)
#define Rule _lib_iv(Rule)
#define SMFKit _lib_iv(SMFKit)
#define SMFKitImpl _lib_iv(SMFKitImpl)
#define ScalingLine _lib_iv(ScalingLine)
#define ScalingLineList _lib_iv(ScalingLineList)
#define ScalingRect _lib_iv(ScalingRect)
#define Scene _lib_iv(Scene)
#define ScrollBar _lib_iv(ScrollBar)
#define ScrollBox _lib_iv(ScrollBox)
#define Scroller _lib_iv(Scroller)
#define SelectionHandler _lib_iv(SelectionHandler)
#define SelectionList _lib_iv(SelectionList)
#define SelectionManager _lib_iv(SelectionManager)
#define SelectionManagerRep _lib_iv(SelectionManagerRep)
#define Sensor _lib_iv(Sensor)
#define Session _lib_iv(Session)
#define SessionRep _lib_iv(SessionRep)
#define Shadow _lib_iv(Shadow)
#define ShadowFrame _lib_iv(ShadowFrame)
#define Shape _lib_iv(Shape)
#define ShowFrame _lib_iv(ShowFrame)
#define SimpleCompositor _lib_iv(SimpleCompositor)
#define Slider _lib_iv(Slider)
#define SliderImpl _lib_iv(SliderImpl)
#define SlidingEllipse _lib_iv(SlidingEllipse)
#define SlidingLine _lib_iv(SlidingLine)
#define SlidingLineList _lib_iv(SlidingLineList)
#define SlidingPointList _lib_iv(SlidingPointList)
#define SlidingRect _lib_iv(SlidingRect)
#define Stencil _lib_iv(Stencil)
#define Stepper _lib_iv(Stepper)
#define StepperSubclassName _lib_iv(StepperSubclassName)
#define StretchingRect _lib_iv(StretchingRect)
#define StringBrowser _lib_iv(StringBrowser)
#define StringChooser _lib_iv(StringChooser)
#define StringEditor _lib_iv(StringEditor)
#define Style _lib_iv(Style)
#define StyleRep _lib_iv(StyleRep)
#define Subject _lib_iv(Subject)
#define SubjectRep _lib_iv(SubjectRep)
#define Superpose _lib_iv(Superpose)
#define TBComposition _lib_iv(TBComposition)
#define TBScrollBox _lib_iv(TBScrollBox)
#define TBScrollBoxImpl _lib_iv(TBScrollBoxImpl)
#define TGlue _lib_iv(TGlue)
#define TIFFRaster _lib_iv(TIFFRaster)
#define TSolver _lib_iv(TSolver)
#define Target _lib_iv(Target)
#define TeXCompositor _lib_iv(TeXCompositor)
#define Telltale _lib_iv(Telltale)
#define TelltaleGroup _lib_iv(TelltaleGroup)
#define TelltaleState _lib_iv(TelltaleState)
#define TextBuffer _lib_iv(TextBuffer)
#define TextButton _lib_iv(TextButton)
#define TextDisplay _lib_iv(TextDisplay)
#define TextEditor _lib_iv(TextEditor)
#define TextRenderInfo _lib_iv(TextRenderInfo)
#define Tile _lib_iv(Tile)
#define TileFirstAligned _lib_iv(TileFirstAligned)
#define TileReversed _lib_iv(TileReversed)
#define TileReversedFirstAligned _lib_iv(TileReversedFirstAligned)
#define TitleFrame _lib_iv(TitleFrame)
#define TopLevelWindow _lib_iv(TopLevelWindow)
#define TransformFitter _lib_iv(TransformFitter)
#define TransformSetter _lib_iv(TransformSetter)
#define Transformer _lib_iv(Transformer)
#define TransformerStack _lib_iv(TransformerStack)
#define TransientWindow _lib_iv(TransientWindow)
#define Tray _lib_iv(Tray)
#define TrayElement _lib_iv(TrayElement)
#define UpMover _lib_iv(UpMover)
#define VBorder _lib_iv(VBorder)
#define VBox _lib_iv(VBox)
#define VGlue _lib_iv(VGlue)
#define VRule _lib_iv(VRule)
#define VScrollBar _lib_iv(VScrollBar)
#define VScroller _lib_iv(VScroller)
#define VariableLayout _lib_iv(VariableLayout)
#define ViewList _lib_iv(ViewList)
#define Viewport _lib_iv(Viewport)
#define WidgetKit _lib_iv(WidgetKit)
#define WidgetKitImpl _lib_iv(WidgetKitImpl)
#define Window _lib_iv(Window)
#define WindowRep _lib_iv(WindowRep)
#define WindowTable _lib_iv(WindowTable)
#define WindowVisual _lib_iv(WindowVisual)
#define World _lib_iv(World)
#define XSlider _lib_iv(XSlider)
#define XYMarker _lib_iv(XYMarker)
#define XYSlider _lib_iv(XYSlider)
#define YSlider _lib_iv(YSlider)
#define Zoomer _lib_iv(Zoomer)
