/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Use 2.6 by default for ambiguous names
 */

#undef Adjuster
#undef Border
#undef Box
#undef Button
#undef PushButton
#undef RadioButton
#undef Center
#undef Coord
#undef Deck
#undef Dialog
#undef FileBrowser
#undef FileChooser
#undef Glue
#undef HGlue
#undef VGlue
#undef Menu
#undef MenuBar
#undef MenuItem
#undef Mover
#undef UpMover
#undef DownMover
#undef LeftMover
#undef RightMover
#undef Panner
#undef Scroller
#undef HScroller
#undef VScroller
#undef ScrollBar
#undef HScrollBar
#undef VScrollBar
#undef Slider

#define Adjuster _lib_iv2_6(Adjuster)
#define Border _lib_iv2_6(Border)
#define Box _lib_iv2_6(Box)
#define Button _lib_iv2_6(Button)
#define PushButton _lib_iv2_6(PushButton)
#define RadioButton _lib_iv2_6(RadioButton)
#define Center _lib_iv2_6(Center)
#define Coord _lib_iv2_6(Coord)
#define Deck _lib_iv2_6(Deck)
#define Dialog _lib_iv2_6(Dialog)
#define FileBrowser _lib_iv2_6(FileBrowser)
#define FileChooser _lib_iv2_6(FileChooser)
#define Glue _lib_iv2_6(Glue)
#define HGlue _lib_iv2_6(HGlue)
#define VGlue _lib_iv2_6(VGlue)
#define Menu _lib_iv2_6(Menu)
#define MenuBar _lib_iv2_6(MenuBar)
#define MenuItem _lib_iv2_6(MenuItem)
#define Mover _lib_iv2_6(Mover)
#define UpMover _lib_iv2_6(UpMover)
#define DownMover _lib_iv2_6(DownMover)
#define LeftMover _lib_iv2_6(LeftMover)
#define RightMover _lib_iv2_6(RightMover)
#define Panner _lib_iv2_6(Panner)
#define Scroller _lib_iv2_6(Scroller)
#define HScroller _lib_iv2_6(HScroller)
#define VScroller _lib_iv2_6(VScroller)
#define ScrollBar _lib_iv2_6(ScrollBar)
#define HScrollBar _lib_iv2_6(HScrollBar)
#define VScrollBar _lib_iv2_6(VScrollBar)
#define Slider _lib_iv2_6(Slider)
