/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef drawclasses_h
#define drawclasses_h

#include <FrameUnidraw/frameclasses.h>
#include <GraphUnidraw/graphclasses.h>

#define DRAW_IDRAW_COMP     9801
#define DRAW_IDRAW_VIEW     Combine(DRAW_IDRAW_COMP,COMPONENT_VIEW)
#define DRAW_IDRAW_PS       Combine(DRAW_IDRAW_COMP,POSTSCRIPT_VIEW)
#define DRAW_IDRAW_SCRIPT   Combine(DRAW_IDRAW_COMP, SCRIPT_VIEW)

#define DRAWLINK_COMP       9802

#define DRAWIMPORT_CMD      9803
#define DRAWIMPORTPASTE_CMD 9804
#define COPYMOVEGRAPHFRAME_CMD 9805

#endif
