#define ClassId _lib_iv(ClassId)
#define Orientation _lib_iv(Orientation)
#define Zooming _lib_iv(Zooming)
#define AcknowledgeDialog _lib_iv(AcknowledgeDialog)
#define AlignCmd _lib_iv(AlignCmd)
#define AlignToGridCmd _lib_iv(AlignToGridCmd)
#define BackCmd _lib_iv(BackCmd)
#define BasicDialog _lib_iv(BasicDialog)
#define BoxObj _lib_iv(BoxObj)
#define BrushCmd _lib_iv(BrushCmd)
#define BrushVar _lib_iv(BrushVar)
#define BrushVarView _lib_iv(BrushVarView)
#define CCnxn _lib_iv(CCnxn)
#define CCnxn_HashTable _lib_iv(CCnxn_HashTable)
#define CGlue _lib_iv(CGlue)
#define CNet _lib_iv(CNet)
#define CSolver _lib_iv(CSolver)
#define CSolverInfo _lib_iv(CSolverInfo)
#define CSolverState _lib_iv(CSolverState)
#define Catalog _lib_iv(Catalog)
#define CatalogChooser _lib_iv(CatalogChooser)
#define CenterCmd _lib_iv(CenterCmd)
#define Clipboard _lib_iv(Clipboard)
#define CloseEditorCmd _lib_iv(CloseEditorCmd)
#define ClosedBSpline _lib_iv(ClosedBSpline)
#define ClosedSplineComp _lib_iv(ClosedSplineComp)
#define ClosedSplineView _lib_iv(ClosedSplineView)
#define ColorCmd _lib_iv(ColorCmd)
#define ColorData _lib_iv(ColorData)
#define ColorVar _lib_iv(ColorVar)
#define Command _lib_iv(Command)
#define CommandControl _lib_iv(CommandControl)
#define CommandInteractor _lib_iv(CommandInteractor)
#define CompNameVar _lib_iv(CompNameVar)
#define CompNameVarView _lib_iv(CompNameVarView)
#define Component _lib_iv(Component)
#define ComponentView _lib_iv(ComponentView)
#define ConfirmDialog _lib_iv(ConfirmDialog)
#define ConnInfo _lib_iv(ConnInfo)
#define ConnectCmd _lib_iv(ConnectCmd)
#define ConnectManip _lib_iv(ConnectManip)
#define ConnectTool _lib_iv(ConnectTool)
#define Connector _lib_iv(Connector)
#define ConnectorView _lib_iv(ConnectorView)
#define ControlInfo _lib_iv(ControlInfo)
#define CopyCmd _lib_iv(CopyCmd)
#define Creator _lib_iv(Creator)
#define CutCmd _lib_iv(CutCmd)
#define Damage _lib_iv(Damage)
#define Data _lib_iv(Data)
#define DataCache _lib_iv(DataCache)
#define DeleteCmd _lib_iv(DeleteCmd)
#define DragManip _lib_iv(DragManip)
#define DupCmd _lib_iv(DupCmd)
#define Editor _lib_iv(Editor)
#define EditorImpl _lib_iv(EditorImpl)
#define EditorInfo _lib_iv(EditorInfo)
#define Ellipse _lib_iv(Ellipse)
#define EllipseComp _lib_iv(EllipseComp)
#define EllipseView _lib_iv(EllipseView)
#define Extent _lib_iv(Extent)
#define ExternView _lib_iv(ExternView)
#define F_Circle _lib_iv(F_Circle)
#define F_ClosedBSpline _lib_iv(F_ClosedBSpline)
#define F_Ellipse _lib_iv(F_Ellipse)
#define F_OpenBSpline _lib_iv(F_OpenBSpline)
#define F_Polygon _lib_iv(F_Polygon)
#define F_Rect _lib_iv(F_Rect)
#define FileNameVarView _lib_iv(FileNameVarView)
#define FillPolygonObj _lib_iv(FillPolygonObj)
#define FixedPin _lib_iv(FixedPin)
#define FontCmd _lib_iv(FontCmd)
#define FontVar _lib_iv(FontVar)
#define FontVarView _lib_iv(FontVarView)
#define FrontCmd _lib_iv(FrontCmd)
#define FullGraphic _lib_iv(FullGraphic)
#define GSData _lib_iv(GSData)
#define GVU_HashTable _lib_iv(GVU_HashTable)
#define GVUpdater _lib_iv(GVUpdater)
#define Graphic _lib_iv(Graphic)
#define GraphicBlock _lib_iv(GraphicBlock)
#define GraphicComp _lib_iv(GraphicComp)
#define GraphicCompTool _lib_iv(GraphicCompTool)
#define GraphicComps _lib_iv(GraphicComps)
#define GraphicView _lib_iv(GraphicView)
#define GraphicViews _lib_iv(GraphicViews)
#define GravityCmd _lib_iv(GravityCmd)
#define GravityVar _lib_iv(GravityVar)
#define GravityVarView _lib_iv(GravityVarView)
#define Grid _lib_iv(Grid)
#define GridCmd _lib_iv(GridCmd)
#define GridDialog _lib_iv(GridDialog)
#define GridSpacingCmd _lib_iv(GridSpacingCmd)
#define GroupCmd _lib_iv(GroupCmd)
#define HPanelControl _lib_iv(HPanelControl)
#define HSlotComp _lib_iv(HSlotComp)
#define HSlotView _lib_iv(HSlotView)
#define HistoryMap _lib_iv(HistoryMap)
#define ImportCmd _lib_iv(ImportCmd)
#define InorderView _lib_iv(InorderView)
#define Iterator _lib_iv(Iterator)
#define KeyMap _lib_iv(KeyMap)
#define Line _lib_iv(Line)
#define LineComp _lib_iv(LineComp)
#define LineObj _lib_iv(LineObj)
#define LineView _lib_iv(LineView)
#define LinkComp _lib_iv(LinkComp)
#define LinkView _lib_iv(LinkView)
#define MacroCmd _lib_iv(MacroCmd)
#define MagnifVar _lib_iv(MagnifVar)
#define MagnifVarView _lib_iv(MagnifVarView)
#define MagnifyTool _lib_iv(MagnifyTool)
#define ManagedWindow _lib_iv(ManagedWindow)
#define ManipGroup _lib_iv(ManipGroup)
#define Manipulator _lib_iv(Manipulator)
#define MatchEditor _lib_iv(MatchEditor)
#define MobilityCmd _lib_iv(MobilityCmd)
#define MobilityData _lib_iv(MobilityData)
#define ModifStatusVar _lib_iv(ModifStatusVar)
#define ModifStatusVarView _lib_iv(ModifStatusVarView)
#define MoveCmd _lib_iv(MoveCmd)
#define MoveData _lib_iv(MoveData)
#define MoveTool _lib_iv(MoveTool)
#define MultiLine _lib_iv(MultiLine)
#define MultiLineComp _lib_iv(MultiLineComp)
#define MultiLineObj _lib_iv(MultiLineObj)
#define MultiLineView _lib_iv(MultiLineView)
#define NOPCmd _lib_iv(NOPCmd)
#define NameMap _lib_iv(NameMap)
#define NameVar _lib_iv(NameVar)
#define NameVarView _lib_iv(NameVarView)
#define NewCompCmd _lib_iv(NewCompCmd)
#define NormSizeCmd _lib_iv(NormSizeCmd)
#define ObjectMap _lib_iv(ObjectMap)
#define ObjectMapElem _lib_iv(ObjectMapElem)
#define OpenBSpline _lib_iv(OpenBSpline)
#define OrientationCmd _lib_iv(OrientationCmd)
#define PSBrush _lib_iv(PSBrush)
#define PSClosedSpline _lib_iv(PSClosedSpline)
#define PSColor _lib_iv(PSColor)
#define PSEllipse _lib_iv(PSEllipse)
#define PSLine _lib_iv(PSLine)
#define PSLink _lib_iv(PSLink)
#define PSMultiLine _lib_iv(PSMultiLine)
#define PSPad _lib_iv(PSPad)
#define PSPattern _lib_iv(PSPattern)
#define PSPin _lib_iv(PSPin)
#define PSPolygon _lib_iv(PSPolygon)
#define PSRaster _lib_iv(PSRaster)
#define PSRect _lib_iv(PSRect)
#define PSSlot _lib_iv(PSSlot)
#define PSSpline _lib_iv(PSSpline)
#define PSStencil _lib_iv(PSStencil)
#define PSText _lib_iv(PSText)
#define PSVertices _lib_iv(PSVertices)
#define PadComp _lib_iv(PadComp)
#define PadGraphic _lib_iv(PadGraphic)
#define PadView _lib_iv(PadView)
#define Page _lib_iv(Page)
#define PanelControl _lib_iv(PanelControl)
#define PanelInteractor _lib_iv(PanelInteractor)
#define PasteCmd _lib_iv(PasteCmd)
#define Path _lib_iv(Path)
#define PatternCmd _lib_iv(PatternCmd)
#define PatternVar _lib_iv(PatternVar)
#define PatternVarView _lib_iv(PatternVarView)
#define Picture _lib_iv(Picture)
#define PinComp _lib_iv(PinComp)
#define PinGraphic _lib_iv(PinGraphic)
#define PinView _lib_iv(PinView)
#define Point _lib_iv(Point)
#define PointObj _lib_iv(PointObj)
#define Polygon _lib_iv(Polygon)
#define PolygonComp _lib_iv(PolygonComp)
#define PolygonView _lib_iv(PolygonView)
#define PostScriptView _lib_iv(PostScriptView)
#define PostScriptViews _lib_iv(PostScriptViews)
#define PostorderView _lib_iv(PostorderView)
#define PreorderView _lib_iv(PreorderView)
#define PrintBS _lib_iv(PrintBS)
#define PrintCmd _lib_iv(PrintCmd)
#define PrintDialog _lib_iv(PrintDialog)
#define QuitCmd _lib_iv(QuitCmd)
#define RasterComp _lib_iv(RasterComp)
#define RasterRect _lib_iv(RasterRect)
#define RasterView _lib_iv(RasterView)
#define RectComp _lib_iv(RectComp)
#define RectView _lib_iv(RectView)
#define RedToFitCmd _lib_iv(RedToFitCmd)
#define RedoCmd _lib_iv(RedoCmd)
#define ReplaceCmd _lib_iv(ReplaceCmd)
#define ReshapeTool _lib_iv(ReshapeTool)
#define RevertCmd _lib_iv(RevertCmd)
#define RotateCmd _lib_iv(RotateCmd)
#define RotateTool _lib_iv(RotateTool)
#define SFH_ClosedBSpline _lib_iv(SFH_ClosedBSpline)
#define SFH_OpenBSpline _lib_iv(SFH_OpenBSpline)
#define SF_Circle _lib_iv(SF_Circle)
#define SF_ClosedBSpline _lib_iv(SF_ClosedBSpline)
#define SF_Ellipse _lib_iv(SF_Ellipse)
#define SF_MultiLine _lib_iv(SF_MultiLine)
#define SF_OpenBSpline _lib_iv(SF_OpenBSpline)
#define SF_Polygon _lib_iv(SF_Polygon)
#define SF_Rect _lib_iv(SF_Rect)
#define S_Circle _lib_iv(S_Circle)
#define S_ClosedBSpline _lib_iv(S_ClosedBSpline)
#define S_Ellipse _lib_iv(S_Ellipse)
#define S_MultiLine _lib_iv(S_MultiLine)
#define S_OpenBSpline _lib_iv(S_OpenBSpline)
#define S_Polygon _lib_iv(S_Polygon)
#define S_Rect _lib_iv(S_Rect)
#define SaveCompAsCmd _lib_iv(SaveCompAsCmd)
#define SaveCompCmd _lib_iv(SaveCompCmd)
#define ScaleCmd _lib_iv(ScaleCmd)
#define ScaleTool _lib_iv(ScaleTool)
#define SelectTool _lib_iv(SelectTool)
#define SlctAllCmd _lib_iv(SlctAllCmd)
#define SlidingPin _lib_iv(SlidingPin)
#define SlotComp _lib_iv(SlotComp)
#define SlotGraphic _lib_iv(SlotGraphic)
#define SlotView _lib_iv(SlotView)
#define SplineComp _lib_iv(SplineComp)
#define SplineView _lib_iv(SplineView)
#define StateVar _lib_iv(StateVar)
#define StateVarView _lib_iv(StateVarView)
#define StencilComp _lib_iv(StencilComp)
#define StencilView _lib_iv(StencilView)
#define StretchTool _lib_iv(StretchTool)
#define TF_2Port _lib_iv(TF_2Port)
#define TF_Direct _lib_iv(TF_Direct)
#define TextBuffer _lib_iv(TextBuffer)
#define TextComp _lib_iv(TextComp)
#define TextGraphic _lib_iv(TextGraphic)
#define TextInteractor _lib_iv(TextInteractor)
#define TextManip _lib_iv(TextManip)
#define TextView _lib_iv(TextView)
#define Tool _lib_iv(Tool)
#define TransferFunct _lib_iv(TransferFunct)
#define UArray _lib_iv(UArray)
#define UChooser _lib_iv(UChooser)
#define UControl _lib_iv(UControl)
#define UControlInteractor _lib_iv(UControlInteractor)
#define UHashElem _lib_iv(UHashElem)
#define UHashTable _lib_iv(UHashTable)
#define ULabel _lib_iv(ULabel)
#define UList _lib_iv(UList)
#define UMap _lib_iv(UMap)
#define UMapElem _lib_iv(UMapElem)
#define UPage _lib_iv(UPage)
#define UStencil _lib_iv(UStencil)
#define UndoCmd _lib_iv(UndoCmd)
#define UngroupCmd _lib_iv(UngroupCmd)
#define Unidraw _lib_iv(Unidraw)
#define VPanelControl _lib_iv(VPanelControl)
#define VSlotComp _lib_iv(VSlotComp)
#define VSlotView _lib_iv(VSlotView)
#define VertexManip _lib_iv(VertexManip)
#define Vertices _lib_iv(Vertices)
#define VerticesComp _lib_iv(VerticesComp)
#define VerticesView _lib_iv(VerticesView)
#define ViewCompCmd _lib_iv(ViewCompCmd)
#define Viewer _lib_iv(Viewer)
#define ViewerView _lib_iv(ViewerView)
#define VoidData _lib_iv(VoidData)
