#undef boolean
#undef u_char
#undef CopyString
#undef Directory
#undef DirectoryImpl
#undef File
#undef FileInfo
#undef Host
#undef InputFile
#undef List
#undef Math
#undef Memory
#undef NullTerminatedString
#undef OutputFile
#undef PtrList
#undef StdInput
#undef StdOutput
#undef String
#undef Table
#undef Table2
#undef UniqueString
#undef UniqueStringPool
#undef UniqueStringTable
