/*
 * 
 * Copyright (c) 2009 Scott E. Johnston
 * Copyright (c) 1996,1999 Vectaport Inc.
 * Copyright (c) 1990, 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * FrameImportCmd - a command for importing frames
 */

#ifndef frame_import_h
#define frame_import_h

#include <OverlayUnidraw/ovimport.h>
#include <Unidraw/Commands/macro.h>

#include <stdio.h>

//: specialized OvImportCmd for importing frames
class FrameImportCmd : public OvImportCmd {
public:
    FrameImportCmd(ControlInfo*, ImportChooser* = nil);
    FrameImportCmd(Editor* = nil, ImportChooser* = nil);

    virtual void Execute();
    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    virtual GraphicComp* Import(const char*);
};

//: specialized PasteCmd for flipbook
class FrameImportPasteCmd : public MacroCmd {
public:
    FrameImportPasteCmd(ControlInfo*, Clipboard* = nil);
    FrameImportPasteCmd(Editor* = nil, Clipboard* = nil);
    virtual ~FrameImportPasteCmd();

    virtual void Execute();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

 protected:
    int _executed;
};

#endif
