mkdir refman3.1
ivdl http://www.ivtools.org/ivtools/doc/refman3.1/error.html refman3.1/error.html
ivdl http://www.ivtools.org/ivtools/doc/refman3.1/index.html refman3.1/index.html
ivdl http://www.ivtools.org/ivtools/doc/refman3.1/left.html refman3.1/left.html
ivdl http://www.ivtools.org/ivtools/doc/refman3.1/refman.html refman3.1/refman.html
