#define Dispatcher _lib_dp(Dispatcher)
#define FdMask _lib_dp(FdMask)
#define IOHandler _lib_dp(IOHandler)
#define RpcHdr _lib_dp(RpcHdr)
#define RpcPeer _lib_dp(RpcPeer)
#define RpcReader _lib_dp(RpcReader)
#define RpcRegistry _lib_dp(RpcRegistry)
#define RpcService _lib_dp(RpcService)
#define RpcWriter _lib_dp(RpcWriter)
#define TimerQueue _lib_dp(TimerQueue)
#define iosb _lib_dp(iosb)
#define iostreamb _lib_dp(iostreamb)
#define istreamb _lib_dp(istreamb)
#define ostreamb _lib_dp(ostreamb)
#define rpcbuf _lib_dp(rpcbuf)
#define rpcstream _lib_dp(rpcstream)
